function [u,v] = gaussianfield2Dspec(N,n)
% Generate an N x N divergence free vector field with Gaussian statistics and power law scaling 
% with an exponent k^n.
%
% Usage:
% [u,v] = gaussianfield2Dspec(N,n);
%
% Inputs:
%   N - size of the field to generate
%   n - spectral power law scaling exponent, i.e. E(k) = k^n
%
% Outputs:
%   u - x-component of velocity
%   v - y-component of velocity
%
% See also: GAUSSIANFIELD2D

r1 = randn(N);r2 = randn(N);
s1 = randn(N);s2 = randn(N);


% flip values so that k -> -k
r1m = flipks(r1);r2m = flipks(r2);
s1m = flipks(s1);s2m = flipks(s2);


[kx,ky] = fmodes2(N,1);
k2 = kx.^2 + ky.^2;
k = sqrt(k2); 


P11 = 1 - kx.*kx./(k2+eps);
P12 = 0 - kx.*ky./(k2+eps);
P21 = 0 - ky.*kx./(k2+eps);
P22 = 1 - ky.*ky./(k2+eps);

u1 = P11.*( r1 + r1m + sqrt(-1)*s1 - sqrt(-1)*s1m ) + ... 
    + P12.*( r2 + r2m + sqrt(-1)*s2 - sqrt(-1)*s2m );

u2 = P21.*( r1 + r1m + sqrt(-1)*s1 - sqrt(-1)*s1m ) + ... 
    + P22.*( r2 + r2m + sqrt(-1)*s2 - sqrt(-1)*s2m );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% alternate fix to problem of mismatched k = -N/2 for N = even
% eliminate all instances where the conjugate pair does not exist
% except (kx,ky) = (N/2,N/2), or where either kx = 0 or ky = 0

% indices to set to zero
inx = (abs(kx) == N/2)&ky~=0;iny = (abs(ky) == N/2)&kx~=0;
in = xor(inx,iny);
in(1,1) = 1; % also eliminate mean

u1(in) = 0;u2(in) = 0;

% generate function to multiply to obtain desired scaling
msk = k.^( (n-1)/2);msk(1,1) = 1;
u1 = u1.*msk;u2 = u2.*msk;

u = ifft2(u1);v = ifft2(u2);


% test divergence (Fourier based)
fd = kx.*u1 + ky.*u2;
d = ifft2(fd); % divergence in real space

mxfd = max(fd(:));mnfd = min(fd(:));
disp(['Max div: ',num2str(real(mxfd)),'  ',num2str(imag(mxfd)),'i']);
disp(['Min div: ',num2str(real(mnfd)),'  ',num2str(imag(mnfd)),'i']);

mxd = max(d(:));mnd = min(d(:));
disp(['Max div physical space: ',num2str(real(mxd)),'  ',num2str(imag(mxd)),'i']);
disp(['Min div physical space: ',num2str(real(mnd)),'  ',num2str(imag(mnd)),'i']);

% Created 8/5/2006 Jori