function X = GaussianCorrNoise(ac, M)
%GAUSSIANCORRNOISE generates a Gaussian noise with given covariance
% 
% Uses the circulant matrix method
%
% Reference:  Fast, exact synthesis of gaussian and nongaussian long
% range dependent processes. Matthew S. Crouse & Richard G. Baraniuk.
% IEEE Transaction on signal processing, 08/99
%
% See article for conditions upon which the algorithm behaves well. It is not
% guaranteed that this will be the case for an arbitrary covariance
% function, but it will work for fractionnal gaussian noise. If no warning
% is displayed, the synthesis is exact (no perdiodization, etc).
%
% Usage :
%   X = GaussianCorrNoise(ac [, M])
%
% Inputs :
%   ac - auto-correlation function as constructed by FGNAutoCorrel
%   M - number of trajectories to generate (default : 1)
%       NB : the algorithm always generates an even number of trajectories
%       internally, so that M = 2 isn't slower than M = 1
%
% Output :
%   X - NxM array of generated trajectories, where N = length(ac)
%
% See also : FGNNoise, FGNCovariance, GaussianCorrNoise
%

% Version 0.1 RNVY 20/06/07


if(nargin < 2)
    M = 1;
end

N = length(ac);

% Correct for parity of M
M2 = M + (mod(M,2));

% Initialize embedding process covariance
eac = [ac, fliplr(ac(2:(N-1)))];

% do fft
lambda = real(fft(eac));

if(any(lambda < 0))
    disp('Warning ! Embedding process has negative power spectrum frequencies !');
    fprintf('Maximum modulus : %d\n', max(abs(lambda(lambda < 0))));
end

lambda = (lambda .* (lambda >= 0));

% Generate complex gaussian white noise
W = randn(2*N-2,M2/2) + i*randn(2*N-2, M2/2);

% Apply linear transformation
lambda = (lambda*(2*N-2)).^(0.5)';
Z = zeros(2*N-2,M2/2);
for k=1:M2/2
    Z(:,k) = lambda .* W(:,k) ;
end

Z = ifft(Z);

X = zeros(N, M2);
X(:, 1:(M2/2)) = real(Z(1:N,:));
X(:, (M2/2+1):M2) = imag(Z(1:N,:));

X = X(:,1:M);
end
