function F = RandWC(N, prior, M, qmf)
%RANDWC generates zero mean signal with random real wc
%
% Will generate wavelet coefficients randomly and then
% construct a signal using IWT_PO, using only one scaling function
% coefficient, which is set to 0 to ensure zero mean.
% Be aware that there are a lot of translation invariance issues because of
% the dyadic grid.
%
% Cf Abramovich, Sapatinas, Silvermann J.Roy.Stat.Soc B 60 (1998)
%
% Usage :
%   F = RandWC(N, prior [,M [,qmf]])
%
% Inputs :
%   N - sampling time
%   prior - function handle to generate the wavelet coefficients
%           prior(i) should generate 2^i coefficients
%   M - number of trajectories (default : 1)
%   qmf - quadrature mirror filter (default : MakeONFilter('Coiflet', 2))
%
% Output :
%   F - N x M array
%
% Example 
%   % to generate a noise with -2 power spectrum slope
%   F = RandWC(1024, @(i)2^(-i)*randn(1,2^i));
%
% See also : FGNNoise, FBMNoise, RandWCBesov

% Version 0.1 RNVY 20/06/07

if(nargin < 3)
    M = 1;
end
if(nargin < 4)
    qmf = MakeONFilter('Coiflet', 2);
end

L = 0;
J = log(N) / log(2);

F = zeros(N, M);

for k=1:M
    W = zeros(1,N);

    for i=L:(J-1)
        W(dyad(i)) = prior(i);
    end;
    
    F(:, k) = IWT_PO(W, L, qmf);
end

end
