function F = RandWCBesov(N, alpha, beta, eta,  M)
%RANDWCBESOV generates zero mean locally regular signal with random real wc
%
% Will generate wavelet coefficients randomly and then
% construct a signal using IWT_PO, using only one scaling function
% coefficient, which is set to 0 to ensure zero mean.
% Be aware that there are a lot of translation invariance issues because of
% the dyadic grid.
% This version uses the following prior :
%
% p(W(j,k)) ~ pi(j)*eta(s(j)W) + (1-pi(j)) delta(W=0);
%   where s(j)^2 is proportional to 2^(-alpha*j) (global smoothness parameter)
%   and pi(j) is proportional to  2^(-beta*j) (sparsity parameter)
%
% Cf Abramovich, Sapatinas, Silvermann J.Roy.Stat.Soc B 60 (1998)
% where it is demonstrated that the signal falls in a certain Besov space
% with probability one, depending on the values for alpha and beta
%
% Usage :
%   F = RandWCBesov(N, alpha, beta [, M [, eta]])
%
% Inputs :
%   N - sampling time
%   alpha - exponant relative to the variance s(j) of the wc
%   beta - exponant relative to the sparsity pi(j) of the wc
%   M - number of trajectories (default : 1)
%   eta - function handle to generate the wavelet coefficients
%         (default : @(n) randn(1,n))
%
% Output :
%   F - N x M array
%
% Example 
%   F = RandWCBesov(1024, 2, 0.5);
%   plot(F);
%
% See also : FGNNoise, FBMNoise, RandWCBesov

% Version 0.1 RNVY 20/06/07

if(nargin < 5)
    M = 1;
end

J = log2(N);
C_1 = N;
C_2 = 2;

qmf = MakeONFilter('Daubechies', 4);

pi = min(1, 2.^(-beta*(0:J))*C_2);
s = sqrt(2.^(-alpha*(0:J)) * C_1);

prior = @(i) ((rand(1, 2^i) < pi(i+1)) .* (s(i+1)*eta(2^i)));

F = RandWC(N, M, prior, qmf);

end
