function [ks,ew,F] = waveletspectrum1D(sig,x)
% Calculate the 1D wavelet spectrum of a signal
%
% Usage:
%   [k,E,F] = waveletspectrum1D(sig,x);
%
% Inputs:
%   sig - signal
%   x   - (optional) x positions, sed to calculate proper wavenumbers
%
% Outputs:
%   k - wavenumbers
%   E - energy
%   F - scale dependent flatness
%

% See also: WAVELETSPECTRUM2D, SPECTRUM1D, SPECTRUM2D, ENERGYSPECTRUM2D

N = length(sig); % size of the 1D domain
n = floor(log2(N));

if ~exist('x','var'); % if the x input is not specified
    x = [0:N-1]/N; % domain size = 1 [0,1)
    L = 1;
    dx = 1;
else
    L = max(x) - min(x); % size of domain
    dx = x(2)-x(1); % spacing
end %if

% Note: we need to make the scales used consistent. Check them for validity
numoct = n; % number of octaves in the signal
nvpo = 8; % number of voices per octave
scales = cwscales(numoct,nvpo); % scales to use, system size = 1
as = cwscale(scales,'sc2a','cmorl');

% take the 1D CWT, one scale/dilation at a time so that we don't encounter
% memory problems
for i=1:length(as);
    [wc,tas] = cwt1d(sig(:)',as(i));
    ew(i) = mean(wc.*conj(wc),2); % wavelet energy spectrum
    if(nargout >= 3)
        F(i) = var(wc.*conj(wc),1,2) / (ew(i)).^2;
    end
end %i

ks = cwscale(as,'a2k'); % convert the dilation parameters used into frequencies
ks = ks/L;

% 2/6/2007 Jori
% Modified 2/13/2007 Jori, changed calculation of spectrum to loop through
% dilations so that we don't run out of memory on the calculation, this
% greatly speeds up the calculation due to reduced memory requirements

% Code cleaned up 11/08/2012 RNVY
