function [wc,as] = cwt1d(ys,as)
%CWT1D Computes the continuous wavelet tranform of a 1D signal
%
% Usage:
%   [wc,as] = cwt1d(ys,as);
%
% Take the CWT of the input signal using the representation of the
% wavelet in physical space or Fourier space
% Uses the complex Morlet wavelet or the Mexican hat,
% depending on the setting in wavelet_toolbox_config.m
%
% Inputs
%   ys - input signal
%   as - dilation parameters 'a' at which to perform the transform
% Outputs
%   wc - wavelet coefficients
%   as - dilation parameters
%
% Example: 
%
%   numoct = 7; % for a signal length 2^7 = 128
%   nvpo = 4;
%   scalesout = cwscales(numoct,nvpo);
%   ks = 1./scalesout; % Fourier wavenumbers (no pi)
%
%   as = cwscale(ks,'k2a');
%   [wc,as] = cwt1d(sig,as); % 1D CWT
%
% See also: CWT2D, CWSCALE, CWSCALES, DRAWCWT1D

% load config file 
wavelet_toolbox_config;
 
% check wavelet
if ~strcmp(wtype,'cmorl') && ~strcmp(wtype,'mexh');
    disp(['Unknown wavelet type ',wtype]);
    return;
end %if

N = length(ys); % length of signal
nscales = length(as); % total number of dilations to take in decomposition
Fys = fft(ys); % Fourier transform of the signal

wc = zeros(nscales,N);

% perform complete decomposition
for j=1:nscales; % loop over scales
% current dilation parameter
    a = as(j);
% generate the wavelet at the current dilation
    if strcmp(wtype,'cmorl')
        Fmo = gen_cmorlet(N,a,'f');
    else
        Fmo = gen_mexh(N,a,'f');            
    end %if
            
    % multiply the Fourier transforms to take the CWT
    Fwc = Fys(:) .* conj( Fmo(:) );

    % store computed wavelet coefficients
    wc(j,:) = ifft(Fwc);
end % loop over scales


% Modified 4/26/2006 Jori
% from CWT_test_P1.m
% Cleaned up code 11/08/2012 RNVY 
