function [coef2D] = cwt2d(champ,sc,theta)
%CWT2D Computed the continuous wavelet tranform of a 2D field
%
% The mother wavelet is the complex Morlet wavelet.
% The orientation of the wavelet is fixed by the parameter theta.
%
% Usage:
%   [coef2D] = cwt2d(champ,sc,theta);
%
% Inputs:
%   champ - field to be analyzed
%   sc - dilation parameters
%   theta - angle of the wavelet
% Outputs:
%   coef2D = 2D CWT coefficients
%
% See also: CWT1D, CWTSCALE, CWSCALES

N = length(champ(:));
FFTfield = fft2(champ);
champ = FFTfield*sqrt(N); % added here sqrt(N) to correct for strange normalization problem 1/10/2007 Jori
nscales = length(sc);

%numberThetas = length(theta); % number of angles to loop over

%for j= 1:numberThetas % loop over angles in the decomposition
    
    for i=1:nscales % loop over scales in the decomposition
        a = sc(i);
    %    disp(['a = ',num2str(a),'; Loop: ',num2str(i),'/',num2str(nscales)]);
    % take the 2D CWT
        [modul2D, phase2D, coef2D(i,:,:)]=wmorlet(champ, a, theta);
    end %i loop over scales in the decomposition

%end %j loop over angles in the decomposition

% Created 5/10/2006 Jori

function [modul, phase, coef]=wmorlet(champ, a, theta)
% [modul, phase, coef]=wmorlet(champ, a, theta);
%
% Updated / optimized to matlab version
%
% Fortran code:     subroutine wmorlet(champ, a, theta, modul, phase)
%
% c
% c  Variables en entree: s      Champ a analyser
% c                       a      Parametre de dilatation
% c                       theta  Angle de rotation
% c
% c  Variables en sortie: modul  Module des coefficients d'ondelettes
% c                       phase  Phase des coefficients d'ondelettes
% c_______________________________________________________________________
% c  entry --->
% c  s:       real array of dimension champ(idim2,idim2,2)
% c           containing the complex fourier tranform of the signal
% c           champ(.,.,1) <--> real part
% c           champ(.,.,2) <--> imaginary part
% c  a:       number of points per cycle (= the scale)
% c  theta:   the rotation angle
% c  ---> exit
% c  modul:   real array of dimension modul(idim2,idim2)
% c           containing the square of the modulus of the transform
% c           linearly scaled between 0 and 1
% c  phase:   real array of dimension modul(idim2,idim2)
% c           containing the phase of the transform
% c           linearly scaled between 0 and 1
% c-------------------------------------------------------------------
% c  Remarque:
% c  L'ondelette est calculee dans l'espace physique, car sa
% c  representation de l'ondelette dans l'espace spectral donnait lieu
% c  a un debordement du domaine spectral quand on contractait
% c--------------------------------------------------------------------

% % wavelet parameters
%     om = 5; % our k_psi
     om = 6; % k_psi, value used in Marie's code
    theta = -theta; % use the inverse of the rotation matrix

    cutoff = 10^-6; % where we set the phase to zero if the modulus is too small
%    cutoff=0.002; % cutoff used in Marie's code; the phase is set to zero if the modulus is too small

% size of domain
%    idim = 128;
    idim = size(champ,1);

% c     Echantillonnage de l'ondelette
% c     _____________________________
% c     L'ondelette est echantillonnee dans l'espace physique
% c     et centree en zero,
% c     la position varie entre -idim2/2 et idim/2(calcul modulo idim/2)

N = idim;

    amin = 2*om/pi/N; % minimum dilation before we should anti-alias
%  % not necessary if we use the representation in Fourier space!!
%     if a < amin; % zero pad
%                 idim = 2*idim;
%                 champ = fft2zeropad(champ,idim);
%                 amin = 2*om/pi/idim; % minimum dilation before we should anti-alias
%             end %if


% compute the wavelet in physical space, centered at zero
% with position from -1/2 to 1/2


% % Replace nested for loops with vectorized versions
%     % modify domain to be [-1/2,1/2]
%     tx=fftshift([-idim/2:idim/2-1]/idim);
%     tx = tx/a; % dilate the wavelet
%     [x,y] = meshgrid(tx);
%     xd=(x*cos(theta)+y*sin(theta));
%     yd=(y*cos(theta)-x*sin(theta));
%     r2=xd.^2+yd.^2;
%     r=exp(-r2/2);
% %    ondel=r.*cos(om*xd)+sqrt(-1)*r.*sin(om*xd);
%     ondel=(1/a)*r.*exp(sqrt(-1)*om*xd);
% 
% 
% % % FFT of the wavelet
% % ondel = fft2(ondel);
% % %ondel(1,1) = 0; % set mean to zero


[ondel,xk] = gen_cmorlet2D(idim,a,theta,'f');
%[ondel,xk] = gen_cmorlet2D(idim,a,theta,'p');

% c--------------------------------------------------------------------
% c     pointwise complex multiplication in frequency space

ondel = conj(ondel).*champ;

% Inverse transform
% c     inverse fouriertransform to finish the convolution

%ondel = ifft2(ondel,N,N);
ondel = ifft2(ondel);

% % 
% % c     splitting in modulus and phase
      modul = abs(ondel).^2;
      re = real(ondel);im = imag(ondel);
          
phi = atan2( im,re ); % theta ranges from [-pi,pi]

%in0 = (im>=0); % upper half plane (Im>=0), needs no adjustments
in1 = (im<0); % lower half plane (Im<0), needs shifting
phi(in1) = phi(in1) + 2*pi;

% phi = atan( im./(re +eps));
% %quadrants
% in1 = (re>0)&(im>0); % 1st quadrant, needs no special treatment
% 
% in2 = (re<0)&(im>0); % 2nd quadrant
% phi(in2) = pi - phi(in2); % pi - |theta|
% 
% in3 = (re<0)&(im<0); % 3rd quadrant
% phi(in3) = pi - phi(in3); % pi + |theta|
% 
% in4 = (re>0)&(im<0); % 4th quadrant
% phi(in4) = 2*pi + phi(in4); % 2pi - |theta|

phase = phi;


% % half axes
% ax1 = (im==0)&(re>0); % Real positive, imaginary 0 axis
% phi(ax1) = 0;
% ax2 = (re==0)&(im>0); % Imaginary positive, real 0 axis
% phi(ax2) = pi/2;
% ax3 = (im==0)&(re<0); % Real negative, imaginary 0 axis
% ph(ax3) = pi;
% ax4 = (re==0)&(im>0); % Imaginary negative, real 0 axis
% phi(ax4) = 3*pi/2;


% c--------------------------------------------------------------------
% c     maximum of the modulus
% c     pour la normalisation

      xmax = max(modul(:));
      
% c--------------------------------------------------------------------
% c     normalisation of the modulus and the phase
% c     pour normaliser avant de tronquer sur 8 bits
% need to set the cutoff value where we set the phase to zero (if the
% modulus is too small)

      modul= modul/xmax;
      phase(modul<cutoff) = 0;
 
      % attempt to fix the discontinuity between phase = 0 and phase = 2pi
      phase((re>0)&(abs(im)<10^-10)&(sign(im)==-1)) = 0;
       
 coef = ondel; % output the coefficients
      
% c--------------------------------------------------------------------
%       return
%       end
% c____________________________________________________________________


% Translated: 4/28/2006 by Jori
% translation of Marie's code last updated 3 Novembre 1989
% Code cleaned up 11/08/2012 RNVY 

end
