function [w,xk] = gen_cmorlet2D(N,sc,theta,type)
% Generation of a 2D Complex Morlet wavelet
%
% Usage:
%   [w,xk] = gen_cmorlet2D(N,a,theta,type);
%
% Use the proper plane-wave modulated by a Gaussian envelope
%
% Returns either the wavelet in physical space or in
% Fourier space, depending upon the input 'type'
%
% Input:
%   N - number of points
%   a - dilation parameter of the wavelet
%   theta - rotation of the wavelet
%   type - 'p' for physical space, 'f' for Fourier space
%
% Output:
%   w - Complex Morlet wavelet
%   xk - 1D positions or frequencies where wavelet is calculated
%
% See also: GEN_CMORLET

%     %% *** conversion between wavelet scale and dilation parameter, adjusted empirically
%     a = 0.1254*sc;
a=sc;

    % frequency of Morlet wavelet
    k_0 = 6;

switch type

    case 'p' % Generate Physical space wavelet
   
    x = [-N/2:N/2 - 1]*(1/N); % periodic domain size = 1, from -1/2 -> 1/2
    xk = x;    
    
    xs = x/a;
    
    [x,y] = meshgrid(xs);
    % rotate
    xd=(x*cos(theta)+y*sin(theta));
    yd=(y*cos(theta)-x*sin(theta));
    r2=xd.^2+yd.^2;
    
    % uncorrected wavelet
%    mo=exp(sqrt(-1)*k_0*xd).*exp(-r2/2);

    % corrected wavelet
    mo = ( exp(sqrt(-1)*k_0*xd) - exp(-k_0^2/2) ) .* exp(-r2/2);
    
    mo=(1/a)*mo; % renormalize due to dilation
    
%     % generate a periodized physical-space wavelet
%         n = [-10:10];
%    x = [0:N - 1]*(1/N); % periodic domain size = 1 from [0,1)
%     
%             for I = 1:length(n);
%         
%                 xs = x/a;
%                 xs = xs + n(I);
%     
%                 mo = ( exp(sqrt(-1)*k_0*xs) - exp(-k_0^2/2) ) .* exp(-xs.^2/2);
%                 mo = (1/sqrt(a))*mo; % renormalize due to scale change
% 
%                 w(:,I) = mo;
%         
%         end %n
   
    
    w = mo; % assign output

    % modified 3/31/2006
    % shift the wavelet to make it appear to be centered at x = 0 for FFT
    % defined over interval 0 - 1
    w = fftshift(mo); % assign output

    
    case 'f' % Generate Fourier space wavelet
% NOT YET IMPLEMENTED IN 2D
        % Frequencies (mode numbers) in wrap-around order
            k = [1:N/2-1];
            k = [0 k -N/2 -fliplr(k)];
            delta = 1/N; % use dx from finer grid??
            fk = k/N/delta; % real frequencies
            k = fk;
            xk = k;            
            
            kd = a*k;               % change scale
            [kx,ky] = meshgrid(kd); % grid of frequencies
        
        % *** ROTATIONS NOT YET INCLUDED
        % Check to make sure this is correct!! 6/2/2006
            % rotate
            kxd=(kx*cos(theta)+ky*sin(theta));
            kyd=(ky*cos(theta)-kx*sin(theta));
       
        
%        kr2 = (2*pi*kx).^2 + (2*pi*ky).^2; 
%        krk02 = (2*pi*kx-k_0).^2 + (2*pi*ky).^2;
        kr2 = (2*pi*kxd).^2 + (2*pi*kyd).^2; % rotated domain
        krk02 = (2*pi*kxd-k_0).^2 + (2*pi*kyd).^2; % rotated domain

        %% uncorrected complex Morlet wavelet in Fourier space
%        fmo = (2*pi) * exp(-kr2/2);

        % corrected complex Morlet wavelet in Fourier space
        fmo = (2*pi)*( exp(-krk02/2) - exp(-k_0^2/2)*exp(-kr2/2) );

        % renormalize due to change in scale ***in 2D
        fmo = a*fmo;

        w = fmo;
        
    otherwise
        disp('Unknown type. Please input either p for physical space or f for Fourier space');
        return;
end

% Created 3/3/2006 Jori