function [w,xk] = gen_mexh(N,L,type)
% Generate a real-valued Mexican Hat wavelet
%
% Usage:
%   [w,xk] = gen_mexh(N,L,type);
%
% Returns either the wavelet in physical space or in
% Fourier space, depending upon the input 'type'
%
% Input:
%   N - number of points
%   L - scale, 1 = no scaling
%   type - 'p' for physical space, 'f' for Fourier space
%
% Output:
%   w - Mexican Hat wavelet
%   xk - positions or frequencies where wavelet is calculated
%

% kpsi = 2*sqrt(2/pi); % the kpsi of the mexican hat

% normalization of the wavelet for unit energy
C = 2/(sqrt(3)*pi^(1/4));

switch type

    case 'p' % Generate Physical space wavelet
   
    % define the domain
        x = [-N/2:N/2 - 1]*(1/N); % periodic domain size = 1 from -1/2 -> 1/2
        xk = x;
   
    % scale the wavelet
        xs = x/L;
    
    % Mexican hat wavelet in physical space
        mo = - C*( xs.^2 - 1) .* exp(-xs.^2/2);
        mo = (1/sqrt(L))*mo; % renormalize due to scale change
        
        w = mo; % assign output

    % shift the wavelet to make it appear to be centered at x = 0 for FFT
    % defined over interval 0 - 1
        w = fftshift(mo); % assign output

    case 'f' % Generate Fourier space wavelet

        % Frequencies (mode numbers) in wrap-around order
        k = [1:N/2-1];
        k = [0 k N/2 -fliplr(k)];
        delta = 1/N; % use dx from finer grid??
        fk = k/N/delta; % real frequencies
        k = fk;
        xk = k;
    
        % change scale
        k = L*k;
        
        % Mexican hat wavelet in Fourier space
        fmo = C*sqrt(2*pi).*(2*pi*k).^2.*exp(-(2*pi*k).^2/2);

        % renormalize due to change in scale
        fmo = sqrt(L)*fmo;
        
        w = fmo;
        
    otherwise
        disp('Please input either "p" for physical space or "f" for Fourier space');
        return;
end

% Created 4/4/2006 Jori