function [FigHandles]=drawcwt1d(wc,ys,sc, mode, bounds)
%DRAWCWT1D Draw the coefficients of the 1D CWT
%
% Usage:
%   [H] = drawCWT1D(wc,ys,as,mode);
%
% Inputs:
%   wc - wavelet coefficients
%   ys - original signal
%   as - dilation parameters use in the CWT
%   mode - one of all, phase, modulus, modulusonly
%
% Outputs:
%   H - handles to the figures, in a two element vector
%       i.e. H(1) is the handle to the phase plot
%            H(2) is the handle to the coefficient plot
%   
%   Creates a plot of the phase and modulus of the coefficients
%   using the pcolor function
%
% See also: ----

% Modified: 3/29/2007: added output figure handles
% Modified : Romain 21/09/07 : cleaned up and fixed invisible figures
% problems. added option to choose plots

%wavelet_toolbox_config;
kpsi = 6;

%Parse mode
dophase = 1; dospectra = 1; dosig = 1; domod = 1;
if(nargin < 4)
    mode = 'all';
end
if(nargin < 5)
    bounds = [0 1];
end
dolog = 0;
doreal = 0;
switch(mode)
    case 'all'
    case 'phase'
        domod = 0;
    case 'modulus'
        dophase =0;
    case 'logmodulus'
        dophase = 0; dolog = 1;
    case 'modulussig'
        dophase = 0; dospectra = 0;
    case 'modulusonly'
        dophase = 0; dosig = 0; dospectra = 0;
    case 'real'
        dophase = 0; domod = 0; doreal = 1;
    otherwise 
        error('Wrong mode : %s', mode);
end


N = length(ys);
xs = (0:(N - 1))*(1/N); % periodic domain from [0,1)
xs = bounds(1) + (bounds(2)-bounds(1))*xs;

% Energy spectrum of signal
[E,k] = spectrum1D(ys,xs); 

% wavelet scalogram
sg = mean(wc.*conj(wc),2);

ks = kpsi/2/pi./sc; % the equivalent wavenumbers that the CWT is calculated at
dxs = N./ks; % scale in terms of dx unit grid spacing
amin = 1.5*kpsi/pi/N; % minimum dilation before we should anti-alias
    
COIdata = load('cmorl_COI.mat');

% convert the COI to units of dxs
COIdata.dxsCOI(2,:) = N./COIdata.COI(2,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PHASE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(dophase)
    % plot the phase of the coefficients
    FigHandles(1) = figure('Name', 'Phase of the wavelet coefficients');
    xlim([bounds(1) bounds(2)]);
    phi = wphase(wc);
    P1 = mypcolor(FigHandles(1),phi);
    
    % plot the original signal
    mysignal(FigHandles(1), P1)
    
    % adjust position of colorbar % this is tough
    % PCB = get(CB,'position');set(CB,'position',[PCB(1)+0.05 (PCB(2)+PCB(4)/2) PCB(3)-0.1 (PCB(4)/4)]);set(CB,'tickdir','out');
    
    % plot the spectral energy
    myspectrum(FigHandles(1), P1);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modulus %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot the modulus of the coefficients
if(domod || doreal)
    if(domod)
        FigHandles(2) = figure('Name', 'Modulus of the wavelet coefficients');
        P1 = mypcolor(FigHandles(2), abs(wc));
    elseif(doreal)
        FigHandles(2) = figure('Name', 'Real part of the wavelet coefficients');
        P1 = mypcolor(FigHandles(2), real(wc));        
    end
    xlim([bounds(1) bounds(2)]);    
     % plot the original signal
    mysignal(FigHandles(2), P1)
    
    % adjust position of colorbar % this is tough
    % PCB = get(CB,'position');set(CB,'position',[PCB(1)+0.05 (PCB(2)+PCB(4)/2) PCB(3)-0.1 (PCB(4)/4)]);set(CB,'tickdir','out');
    
    % plot the spectral energy
    myspectrum(FigHandles(2), P1);

end

function P1 = mypcolor(hfig, mymatrix)
    set(gca ,  'Parent', hfig, ...
         'YScale', 'log', 'YDir', 'reverse', 'YAxisLocation', 'right', 'Box', 'on',...
        'OuterPosition',[0.1 0.0  0.9 0.95], 'TickDir', 'out');
    hold all;
    if(dolog)
        H = pcolor(xs,dxs,log10(mymatrix));
        caxis([-7, -0]);
    else
        H = pcolor(xs,dxs,(mymatrix));
    end
    shading flat;
    axis tight;
    set(H, 'LineStyle', 'none');
    xlabel('Position');
    ylabel('\Delta x (grid spacing)');
    a = ylim;
    set(gca,'YTick', 10.^(ceil(log10(a(1))):1:floor(log10(a(2)))));
    % plot the cone of influence
    hold on;
    %semilogy(bounds(1) + COIdata.COI(1,:)*(bounds(2)-bounds(1)),COIdata.dxsCOI(2,:),'-k', 'LineWidth', 2);
    %L = line([bounds(1) bounds(2)],[N/cwscale(amin,'a2k') N/cwscale(amin,'a2k')]);
    L = line([bounds(1) bounds(2)],[3 3]);
    set(L,'color','k','LineWidth',2);
    cmap = load('mod_colormap.mat');
    colormap(cmap.c);    
    %cb = colorbar('horizontal');    
    P1 = get(gca,'Position'); 
    %set(gca,'Position',[P1(1) P1(2)-0.05 P1(3) P1(4)]);
    %P2 = get(gca,'Position')
    %set(cb,'Position',[P2(1) P2(2)-0.2 P2(3) 0.03]);
   
end

function myspectrum(hfig, P1)
if(dospectra)       
    % plot the spectral energy
    NA2 = axes('Parent', hfig, 'Position', [P1(1)-0.1 P1(2) 0.1 P1(4)], ...
       'XDir','reverse', 'YDir','reverse', 'XScale', 'log', 'YScale', 'log', ...
       'xaxislocation','bottom', 'yticklabel','', 'Color','none','CameraUpVector',[1 0 0] ); % new axis
        hold all;
    plot(NA2, k,E,'k');
    plot(NA2, ks,sg,'*-r'); % plot the wavelet scalogram
    axis tight;
   % axis([min(ks(:)) max(ks(:)) min([E(:);sg(:)]) max([E(:);sg(:)])]);
%   [min([E(:);sg(:)]) max([E(:);sg(:)])*1e-5  ]
%    axis([min(ks(:)) max(ks(:)) max([min([E(:);sg(:)]) max([E(:);sg(:)])*1e-5  ]) max([E(:);sg(:)])]);
    xlim([N/dxs(1) N/dxs(end)]);
    a = xlim;
    set(NA2,'XTick', 10.^(ceil(log10(a(1))):1:floor(log10(a(2)))));
    a = ylim;
    ylim([max(a(2)*1e-20,a(1)),a(2)]);
    a = ylim;
    b = get(NA2,'Position');
    mult = log10(a(2)/a(1)) / b(4);
    a = xlim;
    xlabel(NA2,'wavenumber','rotation', 90,'horizontalalignment','right','verticalalignment','bottom','Position',[4*sqrt(a(2)/a(1)) 10^(mult*0.33)*a(2) 1]);

end
end

function mysignal(hfig, P1)
if(dosig)    
    NA = axes('Parent', hfig, 'position',[P1(1) P1(2)+P1(4) P1(3) 0.1],'xticklabel','' ,'yticklabel','');
    hold all;
    box on;
    rotate(NA,[0 0 1],90);
    plot(xs,ys);
    %if(isempty(bounds))
        axis([min(xs) max(xs) min(real(ys))-0.05*(max(real(ys))-min(real(ys))) max(real(ys))+0.05*(max(real(ys))-min(real(ys)))]);
%    axis([min(xs) max(xs) min(ys)-0.05
    %else
    %    axis([min(xs) max(xs) bounds(1) bounds(2)]);
    %end
end
end

function [phi] = wphase(coeff)
% Calculate the phase of the CWT coefficients
%
% Usage:
%   [phi] = wphase(coeff);
%
% The phase is set to zero if the modulus of the
% coefficients is below a cutoff (10^-6).
%
% Inputs:
%   coeff - the complex wavelet coefficients
%
% Outputs:
%   phi - phase ranging from [0,2pi]

      modul = abs(coeff).^2;
      re = real(coeff);im = imag(coeff);
          
phi = atan2( im,re ); % theta ranges from [-pi,pi]

%in0 = (im>=0); % upper half plane (Im>=0), needs no adjustments
in1 = (im<0); % lower half plane (Im<0), needs shifting
phi(in1) = phi(in1) + 2*pi;


cutoff = 10^-6; % where we set the phase to zero if the modulus is too small
      phi(modul<cutoff) = 0;
 
      % attempt to fix the discontinuity between phase = 0 and phase = 2pi
      phi((re>0)&(abs(im)<10^-10)) = 0;
end



% Created 5/8/2006 Jori

end %function
% Created 5/8/2006 Jori
