%ADAPT_WT_TREE
function strsig = adapt_wt_tree(strsig, G, epsilon)

% rebranch
% algorithm rationale :
% each coefficient should have its two neighbours below a certain
% threshold.
% when a coefficient is found which does not satisfy this condition,
% rebranching is necessary. then we have to find some free space to
% rebranch. 
% we say that a coefficient can be discarded when it is below a secondary
% threshold.
% we keep track of the free space in real time during
% rebranching.
% WE ASSUME THAT EVERY OTHER COEFFICIENT IS A WAVELET COEFFICIENT AT THE
% FINEST SCALE. This means that all zones have an even number of wavelet
% coefficients. In other words, no coefficient in the tree can have a
% single child, only 2 children or no children. (2/0 rule)

if(strsig.level ~= strsig.Jmax)
    error('Input needs to be fully wavelet transformed');
end

sig = strsig.sig;

topo0 = (strsig.topo);
topo1 = (strsig.topo(:,1)-1)/2 + 1;
topo2 = (strsig.topo(:,1)-1)/4 + 1;

% START respacing

% *** 1. 'PASSIF' : locate the places where we must liberate memory. ***
big0 = find(abs(sig(2:2:end)) > epsilon); % room has to be found around these coefficients.
if(isempty(big0))
    return;
end
big = distribute_zones(big0, topo1);
big = cluster_zones(big, G, topo1);
tmpbig = cat(1,big{:});
big = cat(1,tmpbig{:});
big(:,1) = 2*big(:,1)-1;
big(:,2) = 2*big(:,2);

%compute the amount of space that has to be made in high resolution
needed_space = sum(big(:,2) - big(:,1)+1);

while(needed_space)
% *** 2. 'ACTIF' : find the places where we have some available room to meet the requirements of stage 1 ***
small = find(abs(sig(3:4:end)) < epsilon);
small = setdiff(small, interv_to_set(cat(2,(big(:,1)-1)/4+1, big(:,2)/4)));
small = distribute_zones(small, topo2);
small = convert_to_intervals(small);
small(:,1) = 4*small(:,1)-3;
small(:,2) = 4*small(:,2);
% shrink the regions too close to big
for n=1:size(small,1)
    m1 = min(abs(small(n,1) - big(:,2)-1));
    m2 = min(abs(big(:,1) - small(n,2)-1));
    
    if(m1 < 4*G)
        small(n,1) = small(n,1) + 4*G-m1;
    end
    
    if(m2 < 4*G)
        small(n,2) = small(n,2) - (4*G-m2);
    end

end

small = discard_small_zones(small, G);
available_space = floor(sum(small(:,2) - small(:,1)+1)/2);

if(needed_space > available_space)
    error('Not enough available space !!!');
end
% *** 3. 'BUDGET' : compute the transformation to equilibrate PASSIF and ACTIF
shift_indic = zeros(0,2);

% * P1 : suppress a superfluous high resolution zone
hrz = find(diff(topo0(:,2)) == -2);
zshift = 0;
small0 = small;
for iz = hrz'
   z_start = squeeze(topo0(iz,1));
   z_end = squeeze(topo0(iz+1,1)-1);
   smallidx = find((small(:,1) == z_start) & (small(:,2) == z_end),1);
   if(~isempty(smallidx))
       fprintf('Found superflous HRZ.\n');
       % Checking the length of the zone
       z_length  = (z_end - z_start + 1);
       if(z_length/2 <= needed_space)
           % yipee, we can suppress the zone.
           fprintf('Suppressing superflous HRZ.\n');
           shift_indic = cat(1,shift_indic, [z_start, -1 ; z_end+1, 1]);
           needed_space = needed_space - z_length/2;
           %delete the zone
           strsig.topo = strsig.topo([1:iz-1-2*zshift, iz+2-2*zshift:end],:);
           % remove this region from small coefficients
           small0 = small0([1:smallidx-1-zshift, smallidx+1-zshift:end], :);
           zshift = zshift + 1;
       end %if
   end %if
   if(needed_space == 0)
       break;
   end
end %for
topo0 = strsig.topo;
small = small0;

if(needed_space)
% * P2 : shift the frontier of a zone
for iz = 1:size(topo0,1)
    if(topo0(iz,2) == 1)
       smallidx = find(small(:,1) == topo0(iz,1));
       if(~isempty(smallidx))
           z_start = small(smallidx,1);
           z_end = min([z_start + 2*needed_space - 1, 8*floor((small(smallidx,2))/8)]);
           
           if(mod(z_end - z_start+1, 16))
               z_end  = z_end - 8;
           end
           z_length = z_end - z_start + 1;
           if(z_length <= 0)
               continue;
           end
           
           shift_indic = cat(1,shift_indic, [z_start, -1 ; z_end+1, 1]);
           needed_space = needed_space - z_length/2;
           %resize the zone
           strsig.topo(iz,1) = z_end+1;
           small(smallidx,1) = z_end+1 + 2*G;
           fprintf('Shrinking HRZ near %i\n', z_start);
       end
    else
       smallidx = find(small(:,2) == topo0(iz,1)-1);
       if(~isempty(smallidx))
           z_end = small(smallidx,2);
           z_start = max([z_end - 2*needed_space + 1, 8*ceil((small(smallidx,1)-1)/8)+1]);
           if(mod(z_end - z_start+1, 16))
               z_start  = z_start + 8;
           end
           z_length = z_end - z_start + 1;
           if(z_length <= 0)
               continue;
           end
           shift_indic = cat(1,shift_indic, [z_start, -1 ; z_end+1, 1]);
           needed_space = needed_space - z_length/2;
           %resize the zone
           small(smallidx,2) = z_start-2*G;           
           strsig.topo(iz,1) =  z_start;  
           fprintf('Shrinking HRZ near %i\n', z_end+1);           
       end
    end
    if(needed_space == 0)
       break;
    end
end %for
end

topo0 = strsig.topo;

% * P3 : create a new low resolution zone
while(needed_space)
for n = 1:size(big,1)
    % make it as close as possible
    small_right = small((small(:,1) > big(n,2)) & ~ismember(small(:,1),topo0),:);
    small_left = small((big(n,1) > small(:,2)) & ~ismember(small(:,2)+1,topo0),:);
    [vsmall_right, idxsmall_right] = min(small_right(:,1) - big(n,2));
    [vsmall_left, idxsmall_left] = min(big(n,1) - small_left(:,2));    
    
    if(vsmall_right <= vsmall_left)
        z_start = small_right(idxsmall_right,1);
        z_start = ceil((z_start-1)/8)*8+1;
        
        too_close = find(topo0(:,1) - small_right(idxsmall_right,2) < 2*G,1,'last');
        if(topo0(too_close,1) - small_right(idxsmall_right,2) > 0)
            small_right(idxsmall_right,2) = topo0(too_close,1)-2*G-1;
        end
        
        z_end = min(z_start-1 + 2*needed_space, small_right(idxsmall_right,2));
        z_end = floor((z_end)/8)*8;
        if(mod(z_end - z_start+1, 16))
            z_end  = z_end - 8;
        end
        small_idx = (small(:,1) == small_right(idxsmall_right,1));
        small(small_idx,1) = z_start + 2*G;
    else
        z_end = small_left(idxsmall_left,2);        
        z_end = floor((z_end)/8)*8;
        
        too_close = find(-topo0(:,1) + small_left(idxsmall_left,1) < 2*G,1,'last');
        if(-topo0(:,1) + small_left(idxsmall_left,1) > 0)
            small_left(idxsmall_left,1) = topo0(too_close,1)+2*G;
        end
        
        z_start = max(z_end - 2*needed_space+1, small_left(idxsmall_left,1));
        z_start = ceil((z_start-1)/8)*8+1;
        if(mod(z_end - z_start+1, 16))
            z_start  = z_start + 8;
        end
        small_idx = (small(:,2) == small_left(idxsmall_left,2));
        small(small_idx,2) = z_end - 2*G;
    end

    z_length = z_end - z_start + 1;
    
    if(z_length <= 0)
        small = small(~small_idx,:);
        continue;
    end

    if(small(small_idx,2) <= small(small_idx,1)+2*G)
        small = small(~small_idx,:);
    end
    
    if(z_length/2 > needed_space)
        small = small(~small_idx,:); 
        continue;
    end
    
    fprintf('Creating a new LRZ near %i\n', z_start);
    strsig.topo = cat(1, strsig.topo, [z_start, -1 ; z_end+1, 1]);
    shift_indic = cat(1,shift_indic, [z_start, -1 ; z_end+1, 1]);
    needed_space = needed_space - z_length/2;

    if(needed_space == 0) 
        break;
    end %if

end %for
end %while

strsig.topo = sortrows(strsig.topo);

% Finally, use the freed space to put the additional space that was needed.
for n = 1:size(big,1)
    topo0 = strsig.topo;
    z_start1 = (big(n,1));
    z_end1 = big(n,2);
    
    inc_right = (((topo0(:,2) == -1) & (z_start1 == topo0(:,1)))); % increase a zone on its right
    inc_left = (((topo0(:,2) == 1) & (z_end1+1 == topo0(:,1)))); % increase a zone on its left
    
    oups_left = (find((topo0(:,2) == 1) & (z_start1 >= topo0(:,1)) & (z_start1 - topo0(:,1) < 2*G),1));
    oups_right = (find((topo0(:,2) == -1) & (z_end1 < topo0(:,1)) & (topo0(:,1)- z_end1-1 < 2*G),1));
    
        
    if(~isempty(oups_left))
        fprintf('Making room for a new HRZ near %i\n', topo0(oups_left));

    end
    if(~isempty(oups_right))
        fprintf('Making room for a new HRZ near %i\n', topo0(oups_left));

    end
    
    if(~any(inc_right | inc_left)) % create a new HRZ
        fprintf('Creating a new HRZ near %i\n', z_start);
        strsig.topo = cat(1,strsig.topo, [z_start1, 1; z_end1+1,-1]);
    elseif(~any(inc_right))
        fprintf('Growing HRZ near %i\n', z_end1+1);
        strsig.topo(inc_left,1) =  z_start1;
    elseif(~any(inc_left))
        fprintf('Growing HRZ near %i\n', z_start1);
        strsig.topo(inc_right,1) =  z_end1+1;
    else
        fprintf('Suppressing LRZ near %i\n', z_start1);
        strsig.topo = strsig.topo(~inc_right & ~inc_left, :);
        
    end %if

    shift_indic = cat(1,shift_indic,[z_start1, 1 ; z_end1+1, -1]);
        
end  %for n
end %while

strsig.topo = sortrows(strsig.topo);

% *** 4. 'TRANSFERT' : apply the actual changes in memory
strsig = apply_all_shifts(strsig, shift_indic);
%strsig.topo = cat(1, strsig.topo, newtopo);

strsig.Jmax = strsig.J0 + max(cumsum(strsig.topo(:,2)));
strsig.level = strsig.Jmax;

% END respacing
end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
