function [X,N_c,I] = Extraction(X,c)
%EXTRACTION applies global threshold to real wc of a 1D signal or 2D image
%
% Used for extraction of coherent structures in a 1D signal or a 2D image, when the
% incoherent part is a Gaussian white noise.
% Uses FWT_PO/IWT_PO/FWT2_PO/IWT2_PO from Wavelab for the wavelet transforms, and
% ExtractionCore to estimate the variance of the noise.
%
% Usage :
%	[X_s,N_c,I] = Extraction(X,c)
%
% Input :
%	X - array of length N, or matrix of size NxN, representing the signal (N = 2^J)
%	c - quantile parameter to use for thresholding (default: value from wavelet_toolbox_config.m)
%
% Outputs :
%	X_s - extracted coherent part of the signal or image
%	N_c - number of coherent coefficients
%	I   - indices of the coherent coefficients
%
%   Additionnal parameters can be changed in the M-file wsconfig :
%      - gaussian quantile parameter 
%      - coarsest scale of wavelet transform
%      - wavelet filter (qmf)
%
% Example
%   X = 5*((1:1024) < 512) + randn(1,1024);
%   [X_s, N_c] = Extraction(X);
%   figure, plot(X), hold all, plot(X_s);
%
% See also ExtractionDTCWT

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 7/8/12

if(size(X,1) == 1 || size(X,2) == 1 )
	% Gather and check
	[N,J] = dyadlength(X);

	% Read config file
	wavelet_toolbox_config;
	L  = L_1D;
	L_0 = J - scales_out;
	if(nargin < 2)
	    c = c_1dreal;
	end

	% Perform wavelet transform
	X_t = FWT_PO(X, L, qmf(:)');
	     
	% Take care of coarsest scales
	I = ExtractionCore(X_t((2^L+1):(2^L_0)), c);
	% Always keep scaling function coefs
	I = union(I+(2^L), 1:(2^L));
	N_c = numel(I);

	% Set incoherent coefs to 0
	X_t(setdiff(1:N, I)) = 0;
	% Come back to physical space
	X = IWT_PO(X_t, L, qmf(:)');

else if(ndims(X) == 2)

	% Gather and check
	[N,J] = quadlength(X);

	% Read config file
	wavelet_toolbox_config;
	if(nargin < 2)
	    c = c_2dreal;
	end

	if(c < 0)
	    c = sqrt(2*log(N*N));
	end
	L = L_2D;
	K = J - scales_out;

	% Perform wavelet transform
	X_t = FWT2_PO(X, L, qmf(:)');

	% Take care of coarser scales
	I = ExtractionCore(X_t(1:(2^K), 1:(2^K)), c);

	% Always keep scaling function coefs
	X = 0*X_t;
	X(1:(2^L), 1:(2^L)) = 1;
	I = union(I, find(X));
	Iout = I;
	X_t(setdiff(1:numel(X_t), I)) = 0;
	N_c = numel(I);

	% Take care of scales_out finest scales
	% (irrelevant if scales_out == 0)
	for j=K:(J-1)
	    xx = X_t(dyad(j),dyad(j));
	    I = ExtractionCore(xx(:), c);
	    xx(setdiff(1:numel(xx), I)) = 0;
	    X_t(dyad(j),dyad(j)) = xx;
	    N_c = N_c + numel(I);
	    
	    xx = X_t(1:(2^j),dyad(j));
	    I = ExtractionCore(xx(:), c);
	    xx(setdiff(1:numel(xx), I)) = 0;
	    X_t(1:(2^j),dyad(j)) = xx;
	    N_c = N_c + numel(I);
	    
	    xx = X_t(dyad(j),1:(2^j));
	    I = ExtractionCore(xx(:), c);
	    xx(setdiff(1:numel(xx), I)) = 0;
	    X_t(dyad(j),1:(2^j)) = xx;
	    N_c = N_c + numel(I);
	end %for

	% Come back to physical space
	X = IWT2_PO(X_t, L, qmf(:)');
else
	error('X must be a 1D or 2D');
end

end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
