function [I, v_out] = ExtractionCoreL1(X, c, v)
%EXTRACTIONCOREL1 estimates the variance of a noise by iteration with mean absolute deviation
%
% Core function of the iterarive algorithm. This takes a set of numbers
% and tries to extract "outliers" with respect to a gaussian
% distribution with known mean value 0, and unknown variance.
% (A. Azzalini, M. Farge, K. Schneider)
% This version uses mean absolute deviation (L1 norm) instead of empirical variance
% to estimate the variance of the noise. It is faster but less stable than ExtractionCoreL2
%	
% Usage :
%	 [I,v] = ExtractionCoreL1(X , c [, v_0])
%
% Inputs :
%	 X - data, column vector of length N
% 	 c - quantile. if a negative value if given, c will be estimated based on the length of X
%	 v_0 - initial guess and maximum value for the variance (default :
%          total variance of X)
%
% Outputs :
% 	 I - indexes of the outliers
%	 v - variance estimate. equals NaN if the estimate falls above v_0
%
% Example
%   X = randn(1,512);
%   var(X) % ~= 1;
%   X(1) = 100; %include an outlier
%   var(X) % ~= 20
%   [I,v] = ExtractionCoreL1(X,-1) % I = 1, v ~= 1
%
% See also ExtractionCoreL2

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

% tuning
wavelet_toolbox_config;
sec = 1;
X = X(:);

% Adjust c to L1 norm instead of L2 norm
if(isreal(X))
    factor = pi / 2;
else
    factor = 4 / pi;
end
c = c*sqrt(factor);
if(c<0)
	if(isreal(X))
		c = sqrt(2*log(numel(X)))*sqrt(factor);
	else
		c = sqrt(log(numel(X)))*sqrt(factor);    
	end
end


% init
N_1 = numel(X);
I = [];

% total L1 norm
rho_0 = sum(abs(X(:)));

% if no init value is given for the variance, set it to the maximum
% possible value
if(nargin < 3)
    v = rho_0/N_1;
    % else check if we can extract noise with the given variance
else
    v = sqrt(v / factor);
    I = find(abs(X(:)) > v*c);
    N_1 = numel(X) - numel(I);
    % if everything is coherent, we're happy, and stop here
    if(N_1 == 0)
        v_out = 0;
        return;
        % else we might run into trouble
    elseif(~isempty(I))
        % compute variance of the "noise"
        v_new = (rho_0 - sum(abs(X(I))))/N_1;
        % if we're above the maximum admissible, oops
        if((v_new > v*sec))
            % set the estimate to -1 to indicate failure
            v_out = -1;
            % and stop
            return;
        else
            % initialise the variance, and perform the usual
            % iterative algorithm
            v = v_new;
        end
    end
end

% beginning of main loop
for i=1:max_iter
	% find what's above the threshold
	I_new = find(abs(X(:)) > v*c);

	% number of inco. coefs
	N_1 = numel(X) - numel(I_new);

	% new variance estimate
	if(N_1 == 0)
		v = 0;
    elseif(isempty(I_new))
        v = rho_0/N_1;
    else
		v = (rho_0 - sum(abs(X(I_new))))/N_1;
	end

	% check if we've converged
	if(isempty(setxor(I,I_new)))
		break;
	else
		I = I_new;
	end
% end of main loop
end

% check if we've finally converged
if( i == max_iter && (error_if_not_converged) )
	fprintf('The algorithm has failed to converge after %i iterations\n', max_iter);
	error('Giving up.');
end

v_out = factor * v^2;
end

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
