function [X, N_c, I] = ExtractionDTCWT(X, c)
%EXTRACTIONDTCWT applies global threshold to the complex wc of a 1D signal or 2D image
%
% Used for extraction of coherent structures in a 1D signal or 2D image,
% when the incoherent part is a Gaussian white noise.
% Uses dtwavexfm/dtwaveifm by N. Kingsbury for the dual tree complex
% wavelet transform, and ExtractionCore to estimate the variance of the
% noise.
%
% Usage :
%	[X_s,N_c,I] = ExtractionDTCWT(X,c)
%
% Input :
%	X - array of length N representing the signal (N = 2^J)
%	c - quantile parameter used for thresholding (default value in wavelet_toolbox_config.m)

% Outputs :
%	X_s - extracted coherent part of the signal
%	N_c - number of coherent coefficients
%	I   - indices of coherent coefficients
%
%   Additionnal parameters can be changed in the M-file wsconfig :
%      - gaussian quantile parameter c_1dcomplex/c_2dcomplex
%      - coarsest scale of wavelet transform (L_1D/L_2D)
%      - dtcwt filters (biort, qshift)
%
% Example 1
%   X = 5*((1:1024) < 512) + randn(1,1024);
%   [X_s, N_c] = ExtractionDTCWT(X);
%    figure, plot(X,'g'), hold all, plot(X_s,'r');
%
% Example 2
%   [x,y] = meshgrid((-255):256);
%   im = 5*(sqrt(x.^2 + y.^2) < 128) + randn(512); %circle + white noise
%   [im_s, N_c] = Extraction2DDTCWT(im);
%   figure, subplot(1,2,1), imagesc(im), subplot(1,2,2), imagesc(im_s);
%
% See also Extraction

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

if(size(X,1) == 1 || size(X,2) == 1 )
	[N,J] = dyadlength(X);

	% Read config file
	wavelet_toolbox_config;
	nscales = J-L_1D;
	L = L_1D;
	L_0 = J - scales_out;
	if(nargin < 2)
	    c = c_1dcomplex;
	end

	% perform DT-CWT
	[Xl, Xh] = dtwavexfm(X(:), nscales, biort, qshift);

	% convert to wavelab format
	X = WavelabWC(Xl,Xh);

	% Take care of coarsest scales
	I = ExtractionCore(X((2^L+1):(2^L_0)), c);

	% Always keep scaling function coefs
	I = union(I+(2^L), 1:(2^L));

	N_c = 2*2^L + 2*numel(I);

	% set incoherent coefs to 0
	X(setdiff((2^(L+1)):length(X), I)) = 0;

	% come back to DT-CWT format
	[Xl,Xh] = CellWC(X, L);

	% Reconstruct signal in physical space
	X = dtwaveifm(Xl, Xh, biort, qshift);    

	% Transpose
	X = X';
else if(ndims(X) == 2)
	% Gather and check
	[N,J] = quadlength(X);

	% Read config file
	wavelet_toolbox_config;
	nscales = J-L_2D;
	so = scales_out+1;
	n2 = nscales - scales_out;
	if(nargin < 2)
	    c = c_2dcomplex;
	end

	% perform DTCWT
	[Xl, Xh] =  dtwavexfm2(X,nscales,biort,qshift);

	toto = {Xh{so:nscales}};
	% linearize subbands
	for j=1:n2
	    toto{j} = shiftdim(toto{j}, 2);
	    toto{j} = toto{j}(:,:);
	end

	% gather all coefficients in a big vector
	big = cat(2,toto{:});
	mm = size(big);
	% run iterative algorithm on the whole set of coefficients
	I = ExtractionCore(big(:), c);
	% count
	N_c = numel(Xl) + 2*numel(I);
	lookup = cumsum([0 2.^(2*((J-so):-1:3))]);

	% set everything to 0
	for j = 1:n2
	    toto{j}= zeros(size(Xh{j+so-1}));
	end %for

	%recover coherent coefs
	[s,u] = ind2sub(mm,I);
	k = 1;
	for i=I'
	    f = find(u(k) > lookup);
	    j = f(end);
	    u(k) = u(k) - lookup(f(end));
	    [a,b] =ind2sub([2^(J-so+1-j),2^(J-so+1-j)],u(k)); 
	    toto{j}(a,b,s(k)) = Xh{j+so-1}(a,b,s(k));
	    k = k+1;
	end %for


	[Xh{so:nscales}] = deal(toto{:});

	% take care of scales_out finest scales separately
	% (irrelevant if scales_out == 0)
	for j = 1:(so-1)
	   Xh{j} = shiftdim(Xh{j},2);
	   for s = 1:6
		I = ExtractionCore(Xh{j}(s,:), c);
		N_c = N_c + 2*numel(I);
		sz = numel(Xh{j}(s,:));
		Xh{j}(s,setdiff(1:sz,I)) = 0;
	   end %for
	   Xh{j} = shiftdim(Xh{j},1);
	end %for

	% Come back to physical space
	X = dtwaveifm2(Xl, Xh, biort, qshift);

	if(show_figure)
	   %createfigure_subbands(Xh); 
	   dtcwt2d_display(Xl,Xh, 'threshold');
	end

	if(nargout > 2)
	    Iout = cell(nscales, 6);
	    for j=1:nscales
		Iout{j} = find(Xh{j});
	    end
	end


else
	error('X must be 1D or 2D');
end
end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%

