function [X, N_c, vars] = ExtractionProp(X, a_0, c)
%EXTRACTIONPROP applies scale by scale threshold to real wc of a 1D signal or 2D image
%
% Used for extraction of coherent structures in a 1D signal or 2D image when the
% incoherent part is a Gaussian correlated noise with a given maximum
% correlation slope.
% Uses FWT_PO/IWT_PO/FWT2_PO/IWT2_PO from Wavelab for the wavelet transforms, and
% ExtractionCore to estimate the variance of the noise at each scale.
%
%	 [X_s, N_c, vars] = ExtractionProp(X, a_0, c)
%
% Inputs :
%	 X - vector of length N=2^J or matrix of size NxN representing the signal 
% 	a_0 - maximum correlation slope of the noise
%		(log2(W(j,k)) - log2(W(j+1,k)) < a_0)
%	c   - quantile parameter to use for selecting the threshold
%
% Outputs :
%	X_s - extracted coherent part  
% 	N_c - number of coherent coefficients
% 	vars - length J vector of estimated variances, from small scales to
%          large scales (NaN indicates failure to estimate)
%
% Additionnal parameters can be changed in the file wsconfig :
%   - gaussian quantile parameter (c_1dreal/c_2dreal)
%   - coarsest scale of wavelet transform (L_1D/L_2D)
%   - quadrature mirror filter (qmf) 
%
% Example 1
%   X = 5*((1:1024) < 512) + FGNNoise(1024,0.75)';
%   [X_s, N_c] = ExtractionProp(X,1);
%   figure, plot(X,'g'), hold all, plot(X_s,'r');
%
% Example 2
%   [x,y] = meshgrid((-255):256);
%   im = 5*(sqrt(x.^2 + y.^2) < 128) + AFGNNoise2D(512, -.25); %circle + correlated noise
%   [im_s, N_c] = ExtractionProp2D(im,1);
%   figure, subplot(1,2,1), imagesc(im), subplot(1,2,2), imagesc(im_s);
%
% See also ExtractionPropW, ExtractionPropDTCWT

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

if(size(X,1) == 1 || size(X,2) == 1 )

% Read config file
wavelet_toolbox_config;
L = L_1D;
if(nargin < 3)
    c = c_1dreal;
end

% perform wavelet transform
X_t = FWT_PO(X(:), L, qmf(:));

% run iterative algorithm
[I, vars] = ExtractionPropW(X_t, a_0, c, L);

% set incoherent coefficients to 0
X_t(setdiff(1:numel(X), I)) = 0;

% Reconstruct signal in physical space
X = IWT_PO(X_t, L, qmf(:));

% Transposing
X = X';

N_c = numel(I);

else if(ndims(X) == 2)

	% Read config file
	wavelet_toolbox_config;
	L = L_2D;
	if(nargin < 3)
	    c = c_2dreal;
	end

	% perform 2D wavelet transform
	X_t =  FWT2_PO(X,L,qmf(:));

	% convert to cell array ordering
	[Xl,Xh] = CellWC2D(X_t,L);

	% run the iterative algorithm
	[I,vars] = ExtractionPropW2D(Xh, a_0,c);

	% start counting coherent coefs
	N_c = numel(Xl);

	% do the thresholding (a bit awkward)
	for j=1:length(Xh)
		for s = 1:3
		    U = Xh{j}(:,:,s);
		    l = length(U(:));
		    U(setdiff(1:l,I{j,s})) = 0;
		N_c = N_c + numel(I{j,s});
		    Xh{j}(:,:,s) = U;
		end %for
	end %for

	% come back to wavelab ordering
	X_t = WavelabWC2D(Xl,Xh);

	% perform inverse wavelet transform
	X = IWT2_PO(X_t, L, qmf);
else
	error('X must be 1D or 2D');
end
end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
