function [X, N_c, vars] = ExtractionPropDTCWT(X, a_0,c)
%EXTRACTIONPROPDTCWT applies scale by scale threshold to complex wc of a 1D signal or 2D image
%
% Used for extraction of coherent structures in a 1D signal or 2D image when the
% incoherent part is a Gaussian correlated noise with a given maximum
% correlation slope.
% Uses dtwavexfm/dtwaveifm by N. Kingsbury for the dual tree complex wavelet transform,
% and ExtractionCore to estimate the variance of the noise at each scale
%
% Usage :
%	 [X_s, N_c, vars] = ExtractionPropDTCWT(X, a_0, c)
%
% Inputs :
% 	X - vector of length N=2^J representing the signal 
% 	a_0 - maximum correlation slope of the noise
% 	(log2(W(j,k)) - log2(W(j+1,k)) < a_0)
%	c  - quantile parameter used when estimating the threshold (default value in wavelet_toolbox_config.m)
% 
% Outputs : 
% 	X_s - extracted coherent part of the signal 
% 	N_c - number of coherent coefficients
% 	vars - length J vector of estimated variances, from fine scales to
%	 coarse scales (NaN indicates failure to estimate)
%
% Additionnal parameters can be changed in the file wsconfig :
%   - gaussian quantile parameter (c_1dcomplex/c_2dcomplex)
%   - coarsest scale of wavelet transform (L_1D/L_2D)
%   - dtcwt filters (biort, qshift) 
%
% Example 1
%   X = 5*((1:1024) < 512) + FGNNoise(1024,0.75)';
%   [X_s, N_c] = ExtractionPropDTCWT(X,1);
%   figure, plot(X,'g'), hold all, plot(X_s,'r');
%
% Example 2
%   [x,y] = meshgrid((-255):256);
%   im = 5*(sqrt(x.^2 + y.^2) < 128) + AFGNNoise2D(512, -.25); %circle + correlated noise
%   [im_s, N_c] = ExtractionProp2DDTCWT(im,1);
%   figure, subplot(1,2,1), imagesc(im), subplot(1,2,2), imagesc(im_s);
%
%
% See also Extraction, ExtractionProp, ExtractionPropW

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

if(ndims(X) == 1)

	% Gather and check
	[N,J] = dyadlength(X);

	% Read config file
	wavelet_toolbox_config;
	L = L_1D;
	if(nargin < 3)
	    c = c_1dcomplex;
	end

	% perform DT-CWT
	[Xl, Xh] = dtwavexfm(X(:), J-L, biort, qshift);

	% convert to wavelab format
	X_t = WavelabWC(Xl,Xh);

	% run iterative algorithm on wavelet coefficients
	[I, vars] = ExtractionPropW(X_t, a_0, c, L);

	N_c = 2*numel(I);

	% set incoherent coefs to 0
	X_t(setdiff(1:length(X), I)) = 0;

	% come back to DT-CWT format
	[Xl,Xh] = CellWC(X_t, L);

	% Reconstruct signal in physical space
	X = dtwaveifm(Xl, Xh, biort, qshift);    

	% Transpose
	X = X';
else if (ndims(X) == 2)

	% gather and check
	S = 6;
	[N,J] = quadlength(X);

	wavelet_toolbox_config;
	nscales = J-L_2D;
	if(nargin < 3)
	    c = c_2dcomplex;
	end

	% perform DTCWT
	[Xl, Xh] =  dtwavexfm2(X,nscales,biort,qshift);

	% run iterative algorithm
	[I,vars] = ExtractionPropW2D(Xh, a_0,c);
	coh_coef = 0;

	% do the thresholding scale by scale
	for j=1:nscales
		for s = 1:S
		    U = Xh{j}(:,:,s);
		    l = length(U(:));
		coh_coef = coh_coef + 2*numel(I{j,s});
		    U(setdiff(1:l,I{j,s})) = 0;
		    Xh{j}(:,:,s) = U;
		end
	end

	% Count scaling function coefs
	coh_coef = coh_coef + numel(Xl);

	% perform inverse DTCWT
	X = dtwaveifm2(Xl, Xh, biort, qshift);

	if(output_luminy)
	    dtcwt2d_display(Xl, Xh, 'threshold');
	end
else
	error('X must be 1D or 2D');
end

end

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
