function [I,vars] = ExtractionPropW(X_t, a_0, c, L)
%EXTRACTIONPROPW finds coherent 1D wavelet coefficients scale by scale
%
% Extraction algorithm adapted to correlated noise, by initialising the
% variance at each scale by the estimate from next smaller scale,
% corrected by a scaling exponent.
% This low level function works directly on wavelet coefficients, which can be either
% real or complex.
%
% Usage :
%	[I, vars] = ExtractionPropW(X_t, a_0, c, L)
%
% Inputs :
%	 X_t - wavelet coefficients (wavelab format, length(X_t)=2^J)
% 	 L - coarsest scale of wavelet decomposition
% 	a_0 - scaling exponent (log2(W(j,k)) - log2(W(j+1,k)) < a_0) 
%
% Outputs :
% 	I - indexes of coherent coefficients
% 	vars - length J array of estimated variances of the noise at each scale
%	(nan indicate failure to estimate)
%
% Additional parameters can be tuned in the file wsconfig.m :
%	 - the coarsest scale at which we allow estimation of the variance of
%	 the noise (L_0_1D)
%
% See also ExtractionProp, ExtractionPropDTCWT

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

% Gather and check
[N,J] = dyadlength(X_t);

% Read config file
vars = nan*zeros(1,J);
wavelet_toolbox_config;
L_0 = L_0_1D;
j_small = J-1 - scales_out;

for l = 1:scales_out
    vars(l) = 0;
end

if(any(a_0 == -1))
    check_admissibility = 0;
else 
    check_admissibility = 1;
end

% extraction at the smallest scale
[I,vars(1)] = ExtractionCore(X_t(dyad(j_small)), c);
I = I + 2^(j_small);

% extraction scale by scale
for j = (j_small-1):(-1):L_0
	% initialise the variance by a guess from the previous, smaller
	% scale.
	v_next = vars(J-j-1)*(2^a_0);
	% run the extraction algorithm
	[Iplus,vars(J-j)] = ExtractionCore(X_t(dyad(j)), c, v_next);
	% if we didn't find any admissible noise
	if(isnan(vars(J-j)) && check_admissibility)
		L_0 = j;
		% come back to the previous estimate
		v_next = v_next / (2^a_0);
		% then stop estimating
		break;
	end
	% record indexes
	I = union(I, 2^j + Iplus);
end %for

fprintf('ExtractionPropW : stopped at level %i\n', J-L_0);

% Denoise the remaining scales with a fixed variance to avoid instability
if(v_next ~= 0)
    switch(thresh_coarse_scales)
        case 'sure'
            t_ls = 2*ValSUREThresh(X_t(1:(2^L_0))/sqrt(v_next));
        case 'fixed'
            if(c > 0)
                t_ls = c;
            else
                if(isreal(X_t))
                    t_ls = sqrt(2*log(2^L_0));
                else
                    t_ls = sqrt(log(2^L_0));
                end %if
            end %if
        case 'none'
            t_ls = 0;
        otherwise
            error('Wrong config parameter thresh_large_scales = %s', thresh_large_scales);
    end %switch
    Iplus = find(abs(X_t(1:(2^L_0))) > t_ls*sqrt(v_next));
    I = union(I, Iplus);
else
    I = union(I, 1:(2^L_0));
end %if

% Always keep scaling function coefs
I = union(I, 1:(2^L));

% all done
end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
