function [I, vars] = ExtractionPropW2D(Xh, a_0, c)
%EXTRACTIONPROPW2D finds coherent 2D wavelet coefficients scale by scale
%
% Extraction algorithm with variance at each scale initialized before
% the first iteration, to a value depending on the variance at the next
% smallest scale, and on a scaling exponent.
% This low level function works directly on wavelet coefficients. They have
% to be ordered as specified in CellWC2D.
%
% Usage :
%	 [I, vars] = ExtractionPropW2D(Xh, a_0 , c)
%
% Inputs :
%	Xh - cell array of wavelet coefficients
% 	a_0 - vector containing the scaling exponents in each subband. (there
%		are three directions for the DWT, and 6 for the DTCWT)
%		if a_0 is a scalar, we use the same exponant for all subbands.
% 	c - gaussian quantile parameter
%
% Outputs :
% 	I - cell array of size [nscales S] with indexes of coherent coefs
%	vars - matrix of size [nscales S] with estimated variances at each scale and
%		in each subband (NaN indicates failure to estimate)
%
% Additional parameters that can be tuned in wsconfig.m :
%	- coarsest scale allowed for variance estimation (L_0_2D)
%	- show_figure : if nonzero, display a graphical summary of the thresholding
%
% See also ExtractionProp2D, ExtractionProp2DDTCWT

% Version 0.1 RNVY 19/06/07
% Version 1.0 RNVY 07/08/12

% Gather and check
nscales = length(Xh);
[N,M,S] = size(Xh{1});
N = 2*N;
M = 2*M;
J = log2(N);
if((N ~= M) || (J ~= floor(J)) )
    error('Wrong data format');
end
if(length(a_0) ~= S)
    if(length(a_0) == 1)
        a_0 = a_0 * ones(1,S);
    else
        error('Wrong length of second parameter');
    end
end

% Init
I = cell(nscales, S);
vars = nan*zeros(J,S);

% tweaking
wavelet_toolbox_config;
L_0 = max(L_0_2D, J-nscales);

% We need to reshape our arrays, this might be slow...
for j=1:nscales
    Xh{j} = shiftdim(Xh{j},2);
end

% Let's start by the smallest scale.
for s = 1:S
    [I{1,s},vars(1,s)] = ExtractionCore(Xh{1}(s,:),c);
end
if(show_figure)
    infos = cell(1,nscales);
    for j = 1:nscales
        infos{j} = 'Thresholds : ';
    end
    for s = 1:S
        infos{1} = [infos{1}, sprintf('%g /', c*sqrt(vars(1,s)))];
    end
end
% And then go up the scales
for s=1:S
    for j = 2:(J-L_0)
        if(j <= scales_out)
            [Iplus, vars(j,s)] = ExtractionCore(Xh{j}(s,:),c);
        else
            v_next = vars(j-1,s)*2^a_0(s);
            [Iplus, vars(j,s)] = ExtractionCore(Xh{j}(s,:),c,v_next);
        end
        if(isnan(vars(j,s)))
            % the algorithm has detected that the signal
            % dominates over the noise. Come back to the former
            % variance.
            fprintf('ExtractionPropW2D : subband %i, stopped at scale %i\n', s, j);
            vars(j,s) = vars(j-1,s);
            break;
        else
            I{j,s} = Iplus;
            if(show_figure)
                infos{j} = [infos{j}, sprintf('%g /', c*sqrt(vars(j,s)))];
            end
        end
    end
    % If there is some noise remaining
    if(vars(j-1,s) ~= 0)
        % Threshold the remaining scales using given method
        switch(thresh_coarse_scales)
            case 'sure'
                t_ls = 2*ValSUREThresh(catrow(s,Xh{j:nscales})/sqrt(vars(j-1,s)));
            case 'fixed'
                if(c > 0)
                    t_ls = c;
                else
                    if(isreal(Xh{1}))
                        t_ls = sqrt(2*log((N/2^(j-1))^2));
                    else
                        t_ls = sqrt(log((N/2^(j-1))^2));
                    end
                end
            case 'none'
                t_ls = 0;
            otherwise
                error('Wrong config parameter thresh_large_scales = %s', thresh_large_scales);
        end
        for k = j:nscales
            I{k,s} = find(abs(Xh{k}(s,:)) > sqrt(vars(j-1,s))*t_ls);
            infos{k} = [infos{k} sprintf('%g /', sqrt(vars(j-1,s))*t_ls)];
        end
    else
        % Else keep everything
        for k = j:nscales
            I{k,s} = 1:(numel(Xh{k}(s,:)));
            infos{j} = '0 /';
        end
    end

end
% Display thresholded coefficients
if(show_figure)
    % Come back to original ordering
    for j=1:nscales
        Xh{j} = shiftdim(Xh{j},1);
    end
    createfigure_subbands(Xh, I,infos);
end

end

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
