function out_im = NestedSquaresW(X_t, sb)
%NESTEDSQUARESW viewing the 2D real wc using nested squares
%
% Usage :
%   im = NestedSquaresW(X_t, sb)
%
% For the Orthogonal Wavelet Transform, there are three different trees
% (one for each direction). In each tree, every wavelet coefficient has 4
% children at the scale below it. This function will display the parent
% coeffcient as a large square, and then recursively add 4 squares inside
% in the manner of Russian nested dolls.
%
% Inputs :
%    X_t - 2D real wavelet coefficients (wavelab format)
%    sb - subband 1 (xx), 2 (xy) or 3 (yy)
%
% Output :
%   im - wavelet coefficients arrange in a nested squares pattern
%
% Example
%   X = randn(256);
%   X_t = FWT2_PO(X, 3, MakeONFilter('Coiflet',2));
%   imagesc(NestedSquaresW(X_t,1));
%
% See also PlotWCTree, dtcwt2d_display, createfigure_subbands

[N,J] = quadlength(X_t);

if(J < 2)
    error('Matrix size must be a power of 2 larger than 4');
end
out_size = 5*2^(J-2) + 2^(J-2)- 2;
out_im = zeros(out_size);

if(sb == 2)
    start_val = struct('x', 2, 'y', 2);
elseif(sb == 1)
    start_val = struct('x', 2, 'y', 1);
elseif(sb == 3)
    start_val = struct('x', 1, 'y', 2);
else
    error('Non existent subband');
end

tb_add_border(start_val, 1, out_size, 1, out_size);

% Nested function to fill up the borders recusrively
    function tb_add_border(val, haut, bas, gauche, droite)
        out_im(gauche:droite, haut) = X_t(val.x, val.y);
        out_im(gauche:droite, bas) = X_t(val.x, val.y);
        out_im(gauche,haut:bas) = X_t(val.x, val.y);
        out_im(droite,haut:bas) = X_t(val.x, val.y);

        new_size = (droite - gauche - 1)/2;

        [v1, v2, v3, v4] = tb_find_children(val);

        if (new_size > 2)
            tb_add_border(v1, haut+1, haut+new_size, gauche+1,gauche+new_size);
            tb_add_border(v2,haut+1, haut+new_size,droite-new_size,droite-1);
            tb_add_border(v3,bas-new_size,bas-1, gauche+1,gauche+new_size);
            tb_add_border(v4,bas-new_size,bas-1, droite-new_size,droite-1);
        else
            out_im(gauche+1, haut+1) = X_t(v1.x, v1.y);
            out_im(gauche+2, haut+1) = X_t(v2.x, v2.y);
            out_im(gauche+1, haut+2) = X_t(v3.x, v3.y);
            out_im(gauche+2, haut+2) = X_t(v4.x, v4.y);

        end
    end

    function [n_o, n_e, s_o, s_e] = tb_find_children(val)
        %		if((sb == 1) || (sb == 2))
        val.x = 2*(val.x-1)+1;
        %		end
        %		if((sb == 3) ||( sb ==2))
        val.y = 2*(val.y-1)+1;
        %		end
        [n_o, n_e, s_o, s_e] = deal(val);
        n_e.x = n_e.x + 1;
        s_o.y = s_o.y + 1;
        s_e.x = s_e.x + 1;
        s_e.y = s_e.y + 1;

    end
end
