function iso = PlotWCTree(I, L, mode)
%PLOTWCTREE plots a subset of 1D wc as a tree
%
% Coefficients connected to the roots are plotted with symbol 'rx'
% Branches of the tree are plotted in blue.
% When using 'physical' mode, other coefficients are plotted with symbol 'k.'
%  
% Usage :
%   iso = PlotWCTree(I, L, [, mode])
%
% Inputs :
%   I - indexes of chosen coefficients in wavelab format (should be in
%       increasing order)
%   L - coarsest scale of wavelet decomposition
%   mode - 'silent' (default) : do not plot, just calculate iso
%          'physical' : plots the coefficients at their real position on
%                     the space-scale plane
%          'geometric' : makes the tree more easily viewable using 'treelayout'
%
% Output :
%   iso - number of coefficients that are not connected to the roots (isolated) (and
%         thus not displayed on the tree)
%
% Example
%   X = sin(linspace(0, 1, 1024));
%   X_t = FWT_PO(X, 1, MakeONFilter('Coiflet',2));
%   PlotWCTree(find(abs(X_t) > 0.1), 1, 'physical');
%
% See also Extraction

ax = gca;
if(nargin < 3)
    mode = 'silent';
end
doplot = 1;
if(strcmp(mode,'silent'))
    mode = 'geometric';
    doplot = 0;
end

% get rid of scaling function coefficients
I = setdiff(I, 1:(2^L));

switch(mode)
    case 'geometric'
        % Init
        tree = [];
        true_y = [];

        mem = zeros(1, length(I));

        for i=1:length(I)
            j = ceil(log2(I(i)))-1;
            parent = floor((I(i)-1)/2)+1;
            if(parent <= 2^L)
                tree = [tree 0];
                mem(i) = length(tree);
                true_y = [true_y -j];
                continue;
            end
            pp = find(I == parent);
            if(length(pp) > 1)
                error('Index array has duplicate elements');
            end
            if ~(isempty(pp))
                if(mem(pp))
                    tree = [tree mem(pp)];
                    mem(i) = length(tree);
                    true_y = [true_y -j];
                end
            end
        end
        iso = length(I) - length(tree);
        if(~doplot)
            return;
        end
        fprintf('%i coefficients were not connected to the root\n', iso);
        [x, y] = treelayout(tree);
        y = true_y;

        plot(x,y,'rx');hold on;
        for i=1:length(x);
            if(tree(i) ~= 0)
                plot([x(i) x(tree(i))], [y(i), y(tree(i))], 'b-');
            end
        end

    case 'physical'

        x = [];
        y = [];
        x_disco = [];
        y_disco = [];
        mem = zeros(1, length(I));
        tree = [];

        for i=1:length(I)
            j = ceil(log2(I(i)))-1;
            this_x = (I(i) - 2^j) / 2^j - 1/(2^(j+1));
            parent = floor((I(i)-1)/2)+1;
            if(parent <= 2^L)
                x = [x this_x];
                y = [y -j];
                tree = [tree 0];
                mem(i) = length(tree);
                continue;
            end
            pp = find(I == parent);
            if(length(pp) > 1)
                error('Index array has duplicate elements');
            end
            if ~(isempty(pp))
                if(mem(pp))
                    x = [x this_x];
                    y = [y -j];
                    tree = [tree mem(pp)];
                    mem(i) = length(tree);
                end
            else
                x_disco = [x_disco, this_x];
                y_disco = [y_disco, -j];
            end
        end
        iso = length(I) - length(tree);

        fprintf('%i coefficients were not connected to the root\n', iso);
        axes(ax);
        plot(x,y,'rx');hold on;
        plot(x_disco, y_disco, 'k.');
        for i=1:length(tree);
            if(tree(i) ~= 0)
                plot([x(i) x(tree(i))], [y(i), y(tree(i))], 'b-');
            end
        end
        xlim([0 1]);
        ylim([min(y), -L]);

    otherwise
        error('Unknown mode : %s', mode);
end %switch

end %function