%CREATEFIGURE_SUBBANDS displays the wavelet coefficients of a 2D field
%
% The incoherent coefficients are shown in violet. The base 10 logarithm of
% the square modulus of the coherent coefficient is displayed using a continuous colorscale.
% Levels above 6 will be discarded (not enough room on the screen)
% 
% Usage :
%   fig = createfigure_subbands(vh, I, infos)
%
% Inputs :
%   vh - cell array of wavelet coefficients (real or complex), see
%        dtwavexfm2
%   I - cell array of indexes of coherent coefficients
%       size(I) = [nscales, 3] for real wavelets
%       size(I) = [nscales, 6] for complex wavelets
%   infos - cell array of strings to display above the images
%
% Output :
%   fig - handle to an invisible figure
% If no output is requested, the figure will be visible.
%
% Example
%   [vl,vh] = dtwavexfm2(v, 6, 'near_sym_b', 'qshift_b')
%   createfigure_subbands(vh);
%   for j = 1:6
%       for s = 1:6
%            I{j,s} = find(abs(Xh{j}(:,:,s)) > thresh)
%       end
%       infos{j} = sprintf('Threshold = %d', thresh);
%   end
%   fig = createfigure_subbands(vh, I, infos);
%   print(fig, '-dpng', 'example.png');
%
% See also DTCWT2D_display, PaletteInterface, RGBInterface

function figure1 = createfigure_subbands(vh,I,infos)

nscales = length(vh);
if(nscales > 6)
    nscales = 6;
end
    
[N,N,S] = size(vh{1});

if(nargin < 2);
   I = cell(nscales, S);
   for j = 1:nscales
       for s = 1:S
          I{j,s} = find(vh{j}(:,:,s));
       end
   end
end

for j = 1:nscales
   for s = 1:S
   tmp = vh{j}(:,:,s);
   tmp(setdiff(1:numel(tmp), I{j,s})) = nan;
   vh{j}(:,:,s) = tmp;
   end

   vh{j} = vh{j}(:,:);
end

% Create figure
figure1 = figure('XVisual',...
    '0x23 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)',...
    'PaperSize',[20.98 29.68],...
    'Position',   [1     1   637   900],...
    'PaperPositionMode', 'auto', ...
    'InvertHardcopy','off',...
    'Color',[1 1 1]);

ma = zeros(1,6);

% Create axes
for j = 1:nscales
    ma(j) = axes('Parent',figure1,'YTickLabel','','YDir','reverse',...
        'XTickLabel','',...
        'Position',[0 0.84 - (1-0.844)*(j-1) 1 0.122],...
        'Layer','top',...
        'FontSize',12,...
        'FontName','avantgarde',...
        'DataAspectRatio',[1 1 1]);
    box('on');
    hold('all');
    if(nargin >= 3)
        title(sprintf('Level %i: %s',j, infos{j}),'FontSize',12,'FontName','avantgarde');
    else
        title(sprintf('Level %i',j),'FontSize',12,'FontName','avantgarde');        
    end
%     if(j == 1);
%         h = get(ma(j), 'Title');
%         set(h, 'Position', [383, 0]);
%     end
end

minc1 = inf;
maxc1 = 0;
% Create images
for j = 1:nscales
    cdata1 = abs(vh{j}).^2;
    tmpmin = min(cdata1(:));
    tmpmax = max(cdata1(:));
    if(tmpmin < minc1)
        minc1 = tmpmin;
    end
    if(tmpmax > maxc1)
        maxc1 = tmpmax;
    end    
end

C1 = PaletteThreshold(512);

% Create colorbar
ColorbarThreshold(C1, log10(minc1), log10(minc1), log10(maxc1) ,'h',...
    'peer',ma(nscales),[0 0.03 1 0.02], 'Location', 'manual');

% Create images
    minc1 = log10(minc1);
    maxc1 = log10(maxc1);
    
for j = 1:nscales
    vh{j} = log10(abs(vh{j}).^2);
    image(RGBThreshold(vh{j}, minc1, maxc1),'Parent',ma(j));
    axis(ma(j), 'image');
end
end
