function wc = fwt_shannon(ys,pf)
%FWT_SHANNON Computes 1D FWT in Fourier space using periodic Shannon Wavelet
%
% Inputs
%   ys - input signal with size 2^N. ys is supposed to be defined on the domain [0,1[ !!
%
%   pf - 'p' or 'f' if input is in physical or Fourier space (default: p) 
%   For 'f', ys is assumed to be an output of fft, i.e., multiplied by N and with non-negative frequencies first.
%
% Outputs
%   wc - wavelet coefficients 

% Version 1.0 Rodrigo Pereira & RNVY 2010-2012

wavelet_toolbox_config;
type = shannon_or_meyer;

if(nargin < 2)
	pf = 'p';
end

%% Spatial resolution
N=length(ys);
J = round(log2(N));

    switch pf

        case 'f'
            Fys = ys/N;

        case 'p'
	    Fys = fft(ys)/N;

        otherwise 
            disp(['Invalid option: ',pf]);
            return;

    end % switch

wc = [];

% Applying H and G filters recursively

for j=J:-1:1

	n=2^j;

	% filters defined manually
	c = Fys .* conj(h_f(n,type));
	d = Fys .* conj(g_f(n,type));

	% downsampling in Fourier space

	Fys = c(1:n/2) + c((1:n/2)+n/2);
	wt = d(1:n/2) + d((1:n/2)+n/2);

	% keeping G filtered coeffs
	wc = [ifft(wt)*n/2 wc];

end %for j

wc=real([Fys wc]/sqrt(N));

end

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Rodrigo Pereira, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
