function [Yl,Yh] = CellWC(X_t, L)
%CELLWC converts 1D COMPLEX wc from wavelab ordering to cell ordering
%
% The cell array format is 'compatible' with the implementation
% of the DTCWT by N. Kingsbury (dtwavexfm/dtwaveifm). 
% If length(X_t) = 2^J :
%   - the scaling function coefficients are stored in the vector Yl of length 2^L
%   - the wavelet coefficients in the cell array Yh of size [J-L,1]
%   - the cell Yh{l} is a column vector containing the wavelet coefficients
%   at scale j = J-l+1 (corresponding to dyad(J-l)). l is called the LEVEL of wavelet decomposition. 
%
% Usage :
%       [Yl, Yh] = cwt_data(X_t, L)
%
% Inputs :
%	X_t - vector of wavelet coefficients
%	L - coarsest scale of wavelet decomposition
% Outputs :
%  Yl, Yh - coefficients with the same ordering as returned
%           by dtwavexfm
%
% See also WavelabWC

% Version 0.1 RNVY 20/06/07
%

Yl = zeros(2^(L+1),1);

Yl(1:2:(2^(L+1)-1)) = real(X_t(1:2^L));
Yl(2:2:2^(L+1)) = imag(X_t(1:2^L));

J = log2(length(squeeze(X_t.')));

Yh = cell(J-L, 1);
for j = L:(J-1)
Yh{J-j} = X_t(dyad(j)).';

end

end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Rodrigo Pereira, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
