function [Yl, Yh] = CellWC2D(Y_t, L)
%CELLWC2D converts 2D REAL wc from wavelab ordering to cell ordering
%
% The cell array format is 'compatible' with the implementation
% of the DTCWT by N. Kingsbury (dtwavexfm/dtwaveifm). The conversion is
% used by ExtractionProp2D.
%
% If size(Y_t) = [2^J, 2^J], then
% - Yl is a matrix of size [2^L, 2^L] with all the scaling function coefficients  (low pass)
% - Yh is a cell array of size [J-L,1] with all the wavelet coefficients (high pass)
% - Yh{J-j} is an array of size 2^j*2^j*3
%    where the two first dimensions encode for space, and the third for the
%    three possible directions (xx, xy, and yy respectively)
%
% Usage :
% 	[Yl, Yh] = cwt_data2d(Y_t, L)
%
% Inputs :
%	X_t - matrix of wavelet coefficients
%	L - coarsest scale of wavelet decomposition
% Outputs :
%  Yl, Yh - coefficients with the same ordering as returned
%           by dtwavexfm2
%
%
% See also WAVELABWC2D

% Version 0.1 RNVY 20/06/07
%



% gather and check
[N,J] = quadlength(Y_t);

% gather scaling function coefs
Yl = Y_t(1:(2^L), 1:(2^L));

% initialize output
Yh = cell(J-L, 1);

% gather wavelet coefs
for j = L:(J-1)
    Yh{J-j} = zeros(2^j,2^j,3);
    Yh{J-j}(:,:,1) = Y_t(dyad(j), 1:(2^j));
    Yh{J-j}(:,:,2) = Y_t(dyad(j),dyad(j));
    Yh{J-j}(:,:,3) = Y_t(1:(2^j),dyad(j));
end %for

end %function

%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Rodrigo Pereira, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
