function h = daubechies_filters(p,d, type)
%% Calculate the coefficients of Daubechies filters.
% References :
%     - S. Mallat, Une exploration des signaux en ondelettes, Ed. Ecole Pol., p. 246-247
%     - I. Daubechies, Orthonormal bases of compactly supported wavelets,
%       Comm. Pure and Appl. Math, 41, 1988 p.909-996
%
% p = Order of the Daubechies filter
% d = Order of derivation, can be positive or negative, with p >= d
% type = 1 for standard minimal support length daubechies wavelets
% type = 2 for regular coiflettes (only valid for p = 1, p = 2, p = 3)
% ( 1 --> classical daubechies filter).
% The length of the resulting filter is given by 2*p-d
%
if p < d
    error('Must have p >= d');
end

P = zeros(1,p);

for k = 1:p
   P(k) = binomcoef(p-1+p-1-k+1, p-1-k+1); 
end

a = roots(P);
delta = (4*a-2).^2 - 4;
b = -(4*a-2) - sqrt(delta);
c = -(4*a-2) + sqrt(delta);
r = [b/2;c/2];
r = r(abs(r) <= 1);

R = poly(r);

for k = 1:p-d
   R = conv(R, [1/2, 1/2]); 
end

% normalize
renorm = sqrt(P(1)/2 / prod(r)) * 2^(-(p-2));

h = real(R * renorm);

end
%  This file is part of the ENS Wavelet Toolbox 
%  Copyright (C) 2006-2012 LMD-ENS-CNRS-Universite Paris 11 - Romain Nguyen van yen, Rodrigo Pereira, Jori Ruppert-Felsot
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
